package com.zhongshu.reward.server.core.controller.wallet;

import com.github.microservice.auth.security.annotations.ResourceAuth;
import com.github.microservice.auth.security.type.AuthType;
import com.zhongshu.reward.client.model.param.QueryTransferParam;
import com.zhongshu.reward.client.model.param.WalletTransferParam;
import com.zhongshu.reward.client.model.wechat.MiniAppUserInfoVo;
import com.zhongshu.reward.client.ret.ResultContent;
import com.zhongshu.reward.client.type.PlatformType;
import com.zhongshu.reward.server.core.service.WalletReceiptsService;
import com.zhongshu.reward.server.core.service.WalletService;
import com.zhongshu.reward.server.core.service.WxTransferService;
import com.zhongshu.reward.server.core.util.wx.WechatCUtil;
import com.zhongshu.reward.server.core.util.wx.WechatSUtil;
import groovyjarjarpicocli.CommandLine;
import io.swagger.annotations.Api;
import io.swagger.v3.oas.annotations.Operation;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

/**
 * @author wjf
 * @date 2024/8/5
 */
@Slf4j
@RestController
@RequestMapping("/wallet")
@Api(tags = "钱包")
public class WalletController {

    @Autowired
    WalletService walletService;

    @Autowired
    WxTransferService wxTransferService;

    @Autowired
    WalletReceiptsService walletReceiptsService;

    @Autowired
    WechatCUtil wechatCUtil;

    @Autowired
    WechatSUtil wechatSUtil;

    @Operation(summary = "获取（开通）当前用户钱包信息", description = "获取（开通）当前用户钱包信息")
    @ResourceAuth(value = "user", type = AuthType.User)
    @GetMapping ("getWallet")
    public Object getWalletByUser(){
        return walletService.getWallet();
    }

    @SneakyThrows
    @Operation(summary = "获取openid", description = "获取openid")
//    @ResourceAuth(value = "user", type = AuthType.User)
    @GetMapping ("getOpenId")
    public Object getOpenId(@RequestParam("platformType") PlatformType platformType, @RequestParam("jscode") String jscode){

        if (platformType.equals(PlatformType.CLIENT)){
            MiniAppUserInfoVo miniAppUserInfo = wechatCUtil.getMiniAppUserInfo(jscode);
            return ResultContent.buildContent(miniAppUserInfo.getOpenid());
        }else if (platformType.equals(PlatformType.SHOP)){
            MiniAppUserInfoVo miniAppUserInfo = wechatSUtil.getMiniAppUserInfo(jscode);
            return ResultContent.buildContent(miniAppUserInfo.getOpenid());
        }
        return ResultContent.buildFail("没有此平台");
    }

    @Operation(summary = "发起提现", description = "发起提现")
//    @ResourceAuth(value = "user", type = AuthType.User)
    @PostMapping (value = "transfer", consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object transfer(@RequestBody WalletTransferParam param){
        return wxTransferService.transfer(param);
    }


    @Operation(summary = "查询提现列表", description = "查询提现列表")
    @ResourceAuth(value = "user", type = AuthType.User)
    @PostMapping (value = "queryTransfer", consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object queryTransfer(@RequestBody QueryTransferParam param){
        return wxTransferService.queryTransfer(param.getWalletId(), param.getYear(), param.getMonth());
    }

    @Operation(summary = "查询即将到账", description = "查询即将到账")
    @ResourceAuth(value = "user", type = AuthType.User)
    @GetMapping (value = "listByWallet")
    public Object listByWallet(){
        return walletReceiptsService.listByWallet();
    }

    @Operation(summary = "即将到账详情", description = "邀请奖励明细")
    @ResourceAuth(value = "user", type = AuthType.User)
    @GetMapping (value = "receiptsDetail")
    public Object receiptsDetail(@RequestParam("receiptsId") String receiptsId){
        return walletReceiptsService.receiptsDetail(receiptsId);
    }


}
