package com.zhongshu.reward.server.core.dao.Impl;

import com.github.microservice.components.data.mongo.mongo.helper.DBHelper;
import com.zhongshu.reward.client.model.TotalStatisticsModel;
import com.zhongshu.reward.client.type.TransferStatus;
import com.zhongshu.reward.server.core.dao.extend.WxTransferBatchDaoExtend;
import com.zhongshu.reward.server.core.domain.WxTransferBatch;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wjf
 * @date 2024/8/5
 */
public class WxTransferBatchDaoImpl implements WxTransferBatchDaoExtend {

    @Autowired
    MongoTemplate mongoTemplate;

    @Autowired
    DBHelper dbHelper;


    @Override
    public List<WxTransferBatch> list(String walletId, Long startTime, Long endTime) {
        Criteria criteria = new Criteria();

        if (ObjectUtils.isNotEmpty(walletId)){
            criteria.and("wallet._id").is(walletId);
        }
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));
//        if (null != startTime){
//            criteria.and("createTime").gte(startTime);
//        }
//
//        if (null != endTime){
//            criteria.and("createTime").lte(endTime);
//        }

        Query query = Query.query(criteria);
        query.with(Sort.by(Sort.Order.desc("createTime")));

        return mongoTemplate.find(query, WxTransferBatch.class);
    }

    @Override
    public boolean updateStatus(String id, String batchStatus, String detailStatus, TransferStatus transferStatus) {
        Query query = Query.query(Criteria.where("_id").is(id));
        Update update = new Update();
        update.set("batchStatus", batchStatus);
        update.set("detailStatus", detailStatus);
        update.set("transferStatus", transferStatus);
        this.dbHelper.updateTime(update);
        return this.mongoTemplate.updateFirst(
                query, update, WxTransferBatch.class
        ).getModifiedCount() > 0;
    }

    public Integer countByTime(String walletId, Long startTime, Long endTime) {
        Criteria criteria = new Criteria();
        criteria.and("wallet._id").is(walletId);
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));
//        criteria.and("createTime").gte(startTime);
//        criteria.and("createTime").lte(endTime);
        criteria.and("transferStatus").in(List.of(TransferStatus.PROCESSING, TransferStatus.SUCCESS));

        Query query = Query.query(criteria);
        query.with(Sort.by(Sort.Order.desc("createTime")));
        List<WxTransferBatch> wxTransferBatches = mongoTemplate.find(query, WxTransferBatch.class);
        if (wxTransferBatches.isEmpty()){
            return 0;
        }
        return wxTransferBatches.size();
    }

    public Integer sumDayTotal(String walletId, Long startTime, Long endTime){

        Criteria criteria = new Criteria();
        criteria.and("wallet._id").is(walletId);
        criteria.andOperator(Criteria.where("createTime").gte(startTime),Criteria.where("createTime").lte(endTime));
//        criteria.and("createTime").gte(startTime);
//        criteria.and("createTime").lte(endTime);
        criteria.and("transferStatus").in(List.of(TransferStatus.PROCESSING, TransferStatus.SUCCESS));

        Aggregation aggregation = Aggregation.newAggregation(
                Aggregation.match(criteria),
//                Aggregation.project("_id","amountTotal"),
                Aggregation.group("wallet._id").sum("total").as("totalAmount")
//                Aggregation.group("tradeType").last("tradeType").as("_id")
        );


        AggregationResults<TotalStatisticsModel> groupList = mongoTemplate.aggregate(aggregation, WxTransferBatch.class, TotalStatisticsModel.class);

        if (groupList.getMappedResults().size()==0){
            return 0;
        }
        return groupList.getMappedResults().get(0).getTotalAmount();
    }

}
